package odern::Perl;
#ABSTRACT: Module for enabling all of the features of Modern Perl
$odern::Perl::VERSION = '1.20250607';
use Modern::Perl;

*import = \&Modern::Perl::import;

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

odern::Perl - Module for enabling all of the features of Modern Perl

=head1 VERSION

version 1.20250607

=head1 AUTHOR

chromatic

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2025 by chromatic@wgz.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
